import type ts from 'typescript';
export interface PluginConfig {
    [x: string]: any;
    /**
     * Language Server TypeScript Plugin name
     */
    name?: string;
    /**
     * Path to transformer or transformer module name
     */
    transform?: string;
    /**
     * tsconfig.json file (for transformer)
     */
    tsConfig?: string;
    /**
     * The optional name of the exported transform plugin in the transform module.
     */
    import?: string;
    /**
     * Plugin entry point format type, default is program
     */
    type?: 'ls' | 'program' | 'config' | 'checker' | 'raw' | 'compilerOptions';
    /**
     * Apply transformer after internal TypeScript transformers
     */
    after?: boolean;
    /**
     * Apply transformer on d.ts files
     */
    afterDeclarations?: boolean;
    /**
     * Transform *Program* instance (alters during createProgram()) (`type`, `after`, & `afterDeclarations` settings will
     * not apply) Entry point must be (program: Program, host?: CompilerHost) => Program
     */
    transformProgram?: boolean;
    /**
     * Alias to transformProgram
     * @deprecated
     */
    beforeEmit?: boolean;
}
export declare type TransformerList = Required<ts.CustomTransformers>;
export declare type TransformerPlugin = TransformerBasePlugin | ts.TransformerFactory<ts.SourceFile>;
export declare type PluginFactory = LSPattern | ProgramPattern | ConfigPattern | CompilerOptionsPattern | TypeCheckerPattern | RawPattern;
export interface TransformerBasePlugin {
    before?: ts.TransformerFactory<ts.SourceFile>;
    after?: ts.TransformerFactory<ts.SourceFile>;
    afterDeclarations?: ts.TransformerFactory<ts.SourceFile | ts.Bundle>;
}
export declare type DiagnosticMap = WeakMap<ts.Program, ts.Diagnostic[]>;
export declare type TransformerExtras = {
    /**
     * Originating TypeScript instance
     */
    ts: typeof ts;
    /**
     * TypeScript library file event was triggered in (ie. 'tsserverlibrary' or 'typescript')
     */
    library: string;
    addDiagnostic: (diag: ts.Diagnostic) => number;
    removeDiagnostic: (index: number) => void;
    diagnostics: readonly ts.Diagnostic[];
};
export declare type ProgramTransformerExtras = {
    /**
     * Originating TypeScript instance
     */
    ts: typeof ts;
};
export declare type ProgramTransformer = (program: ts.Program, host: ts.CompilerHost | undefined, config: PluginConfig, extras: ProgramTransformerExtras) => ts.Program;
export declare type LSPattern = (ls: ts.LanguageService, config: {}) => TransformerPlugin;
export declare type CompilerOptionsPattern = (compilerOpts: ts.CompilerOptions, config: {}) => TransformerPlugin;
export declare type ConfigPattern = (config: {}) => TransformerPlugin;
export declare type TypeCheckerPattern = (checker: ts.TypeChecker, config: {}) => TransformerPlugin;
export declare type ProgramPattern = (program: ts.Program, config: {}, extras: TransformerExtras) => TransformerPlugin;
export declare type RawPattern = (context: ts.TransformationContext, program: ts.Program, config: {}) => ts.Transformer<ts.SourceFile>;
